@@ -4,7 +4,7 @@
  * Licensed GPL v2
  * http://PacketSender.com/
  *
- * Copyright Dan Nagle
+ * Copyright NagleCode, LLC
  *
  */
 #include "mainwindow.h"
@@ -28,6 +28,7 @@
 #include <QSslKey>
 #include <QUrl>
 #include <QUrlQuery>
+#include <QPlainTextEdit>
 
 #include <QJsonArray>
 #include <QJsonDocument>
@@ -35,13 +36,24 @@
 
 #include <QStringList>
 
+#if QT_VERSION < QT_VERSION_CHECK(5, 10, 0)
+
+#else
+#include <QRandomGenerator>
+#endif
+
+
+#include <QStandardPaths>
 
 #include "brucethepoodle.h"
+#include "irisandmarigold.h"
 #include "settings.h"
 #include "about.h"
 #include "subnetcalc.h"
 #include "udpflooding.h"
 #include "cloudui.h"
+#include "postdatagen.h"
+#include "panelgenerator.h"
 
 
 int hexToInt(QChar hex);
@@ -49,17 +61,6 @@ void parserMajorMinorBuild(QString sw, unsigned int &major, unsigned int &minor,
 extern void themeTheButton(QPushButton * button);
 
 
-//Only AppImage linux needs to check for updates.
-
-#ifdef __linux__
-
-#define SNAPBUILD true
-
-#else
-
-#define SNAPBUILD false
-
-#endif
 
 MainWindow::MainWindow(QWidget *parent) :
     QMainWindow(parent),
@@ -79,10 +80,9 @@ MainWindow::MainWindow(QWidget *parent) :
     setWindowIcon(mIcon);
 
     tableActive = false;
+    darkMode = settings.value("darkModeCheck", true).toBool();
 
-    //seed qrand
-    QTime time = QTime::currentTime();
-    qsrand( static_cast<unsigned int>(time.msec()));
+    PanelGenerator::darkMode = darkMode;
 
     maxLogSize = 10000;
 
@@ -90,9 +90,8 @@ MainWindow::MainWindow(QWidget *parent) :
         maxLogSize = 100;
     }
 
-#if IS_STUDIO
+
     ui->generatePanelButton->hide();
-#endif
 
     http = new QNetworkAccessManager(this); //Main application http object
 
@@ -117,6 +116,7 @@ MainWindow::MainWindow(QWidget *parent) :
     // default is TCP
     ui->udptcpComboBox->setCurrentIndex(ui->udptcpComboBox->findText("TCP"));
 
+
     //load last session
     if (settings.value("restoreSessionCheck", true).toBool()) {
         QDEBUG() << "Restoring last session";
@@ -187,10 +187,23 @@ MainWindow::MainWindow(QWidget *parent) :
 //   statusBar()->insertPermanentWidget(0, generatePSLink());
 //   statusBar()->insertPermanentWidget(1, generateDNLink());
 
+    // handle double-clicking the ASCII window
+    //asciiPreviewFilter = new PreviewFilter{ui->packetASCIIEdit, true};
+    //hexPreviewFilter = new PreviewFilter{ui->packetHexEdit, false};
 
+    asciiPreviewFilter = new PreviewFilter{ui->packetASCIIEdit, ui->packetASCIIEdit, ui->packetHexEdit};
+    hexPreviewFilter = new PreviewFilter{ui->packetHexEdit, ui->packetASCIIEdit, ui->packetHexEdit};
+
+    if (!connect(asciiPreviewFilter, &PreviewFilter::asciiUpdated, this, &MainWindow::on_packetASCIIEdit_lostFocus)) {
+        QDEBUG() << "asciiPreviewFilter connection false";
+    }
+
+    if (!connect(hexPreviewFilter, &PreviewFilter::hexUpdated, this, &MainWindow::on_packetHexEdit_lostFocus)) {
+        QDEBUG() << "hexPreviewFilter connection false";
+    }
 
     stopResendingButton = new QPushButton("Resending");
-    stopResendingButton->setStyleSheet("QPushButton { color: black; } QPushButton::hover { color: #BC810C; } ");
+    stopResendingButton->setStyleSheet(PersistentConnection::RESEND_BUTTON_STYLE);
     themeTheButton(stopResendingButton);
     stopResendingButton->setIcon(QIcon(PSLOGO));
 
@@ -235,9 +248,6 @@ MainWindow::MainWindow(QWidget *parent) :
 
     statusBar()->insertPermanentWidget(4, sslServerStatus);
 
-    IPv4Stylesheet = "QPushButton {width:75px; color: lightgreen; } QPushButton::hover { color: #BC810C; } ";
-    IPv6Stylesheet = "QPushButton {width:75px; color: lightblue; } QPushButton::hover { color: #BC810C; } ";
-
     //ipmode toggle
     IPmodeButton = new QPushButton("IPv4 Mode");
     themeTheButton(IPmodeButton);
@@ -246,6 +256,7 @@ MainWindow::MainWindow(QWidget *parent) :
     setIPMode();
 
 
+
     connect(IPmodeButton, SIGNAL(clicked()),
             this, SLOT(toggleIPv4_IPv6()));
 
@@ -284,11 +295,19 @@ MainWindow::MainWindow(QWidget *parent) :
 
 
 
-    //Bruce is my pet poodle.
+    //Bruce was my wonderful poodle rescue. 2008-2021
     //Dog easter egg.  CTRL D, O, G.
     //             or  CMD D, O, G.
-    QShortcut *shortcut = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_D, Qt::CTRL + Qt::Key_O, Qt::CTRL + Qt::Key_G), this);
-    QDEBUG() << ": dog easter egg Connection attempt " << connect(shortcut, SIGNAL(activated()), this, SLOT(poodlepic()));
+    QShortcut *shortcutDog = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_D, Qt::CTRL + Qt::Key_O, Qt::CTRL + Qt::Key_G), this);
+    QDEBUG() << ": dog easter egg Connection attempt " << connect(shortcutDog, SIGNAL(activated()), this, SLOT(poodlepic()));
+
+
+    //Iris and Marigold are my rescue puppies.
+    //Pup easter egg.  CTRL P, U, P.
+    //             or  CMD P, U, P.
+    QShortcut *shortcutPup = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_P, Qt::CTRL + Qt::Key_U, Qt::CTRL + Qt::Key_P), this);
+    QDEBUG() << ": puppy easter egg Connection attempt " << connect(shortcutPup, SIGNAL(activated()), this, SLOT(puppypic()));
+
 
     QShortcut *field1 = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_1), this);
     QShortcut *field2 = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_2), this);
@@ -380,6 +399,32 @@ MainWindow::MainWindow(QWidget *parent) :
 
     //on_actionExport_Packets_JSON_triggered();
 
+
+
+    // QTimer::singleShot(100.0, [this](){ this->timeout();});
+
+    if(settings.value("autolaunchStarterPanelButton", false).toBool()) {
+        statusBarMessage("Auto-launching starter panel.", 3000, true);
+
+    }
+
+    // Delayed to give message time to display, and make sure this is on top.
+    QTimer::singleShot(1500, this, [this] () {
+        QSettings settings(SETTINGSFILE, QSettings::IniFormat);
+        if(settings.value("autolaunchStarterPanelButton", false).toBool()) {
+            QDEBUG() << "Auto-launching starter panel";
+            this->on_actionPanel_Generator_triggered();
+        }
+
+    } );
+
+
+
+
+
+
+
+
     QDEBUG() << "Settings file loaded" << SETTINGSFILE;
     QDEBUG() << "Packets file loaded" << PACKETSFILE;
 
@@ -430,18 +475,6 @@ void MainWindow::generateConnectionMenu()
 
 
 }
-/*
-
-{"githubpath":"https:\/\/api.github.com\/repos\/dannagle\/PacketSender\/releases\/7612134",
-"windowsversion":"v5.4.2","macversion":"v5.4.2","linuxversion":"v5.4.2","windowsdownload":
-"https:\/\/github.com\/dannagle\/PacketSender\/releases\/download\/v5.4.2\/PacketSender_5_4_2_2017-09-01.exe",
-"windowsportable":"https:\/\/github.com\/dannagle\/PacketSender\/releases\/download\/v5.4.2\/PacketSenderPortable_5_4_2_2017-09-01.zip",
-"macdownload":"https:\/\/github.com\/dannagle\/PacketSender\/releases\/download\/v5.4.2\/PacketSender_v5_4_2_2017-09-01.dmg","linuxdownload":
-"https:\/\/github.com\/dannagle\/PacketSender\/releases\/download\/v5.4.2\/PacketSenderLinux_5_4_2_2017-09-01.AppImage"}
-
-
-*/
-
 
 void parserMajorMinorBuild(QString sw, unsigned int & major, unsigned int & minor, unsigned int & build)
 {
@@ -473,6 +506,11 @@ void MainWindow::updateManager(QByteArray response)
 {
     QSettings settings(SETTINGSFILE, QSettings::IniFormat);
 
+    //snaps do not need to check for updates.
+#ifdef ISSNAP
+    return;
+#endif
+
     if (!response.isEmpty()) {
         QJsonDocument doc = QJsonDocument::fromJson(response);
         if (!doc.isNull()) {
@@ -547,28 +585,68 @@ void MainWindow::updateManager(QByteArray response)
         return;
     }
 
-    if(! (SNAPBUILD)) {
-        //snaps do not need to check for updates.
+    if (!settings.value("checkforUpdatesAsked", false).toBool()) {
+        settings.setValue("checkforUpdatesAsked", true);
+        settings.setValue("SW_VERSION", SW_VERSION); // first run. Save the current version.
+        QMessageBox msgBox;
+        msgBox.setWindowIcon(QIcon(":pslogo.png"));
+        msgBox.setWindowTitle("Updates.");
+        msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
+        msgBox.setDefaultButton(QMessageBox::Yes);
+        msgBox.setIcon(QMessageBox::Warning);
+        msgBox.setText("Let Packet Sender check for updates weekly?");
+        int yesno = msgBox.exec();
+        if (yesno == QMessageBox::Yes) {
+            QDEBUG() << "Will check for updates";
+            settings.setValue("checkforUpdates", true);
+        } else {
+            QDEBUG() << "Will NOT check for updates";
+            settings.setValue("checkforUpdates", false);
+        }
 
-        if (!settings.value("checkforUpdatesAsked", false).toBool()) {
-            settings.setValue("checkforUpdatesAsked", true);
+        settings.sync();
+    } else {
+
+        //This is not the first run.
+        QString previousVersion = settings.value("SW_VERSION", "unknown").toString();
+        QString swCheck = SW_VERSION;
+
+        // remove v prefix (not used in all installations)
+        previousVersion.replace("v", "");
+        swCheck.replace("v", ""); // remove v prefix (not used in all installations)
+
+        // We only pop the update prompt if it is a newer version we haven't seen.
+        QStringList previousList = previousVersion.split(".");
+        QStringList swList = swCheck.split(".");
+        int swListCount = 1;
+        int prevListCount = 0;
+        if((swList.size() == 3) && (previousList.size() == 3)) {
+            swListCount = swList[0].toUInt() * 10000 + swList[1].toUInt() * 100 + swList[2].toUInt();
+            prevListCount = previousList[0].toUInt() * 10000 + previousList[1].toUInt() * 100 + previousList[2].toUInt();
+            if(swListCount != prevListCount) {
+                if(swListCount > prevListCount) {
+                    QDEBUG() << "New version unseen before. We pop the prompt!";
+                } else {
+                    QDEBUG() << "This is an older version. Do not prompt.";
+                }
+            }
+        }
+
+
+        if((swListCount > prevListCount) && (previousVersion != swCheck)) {
+            QDEBUG() << "New version detected:" << previousVersion << "!=" << swCheck;
+            settings.setValue("SW_VERSION", swCheck);
             QMessageBox msgBox;
             msgBox.setWindowIcon(QIcon(":pslogo.png"));
-            msgBox.setWindowTitle("Updates.");
+            msgBox.setWindowTitle("Packet Sender Updated!");
             msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
             msgBox.setDefaultButton(QMessageBox::Yes);
             msgBox.setIcon(QMessageBox::Warning);
-            msgBox.setText("Let Packet Sender check for updates weekly?");
+            msgBox.setText("Updated to " + swCheck + "!\n\nWould you like to read the release notes?");
             int yesno = msgBox.exec();
             if (yesno == QMessageBox::Yes) {
-                QDEBUG() << "Will check for updates";
-                settings.setValue("checkforUpdates", true);
-            } else {
-                QDEBUG() << "Will NOT check for updates";
-                settings.setValue("checkforUpdates", false);
+                QDesktopServices::openUrl(QUrl("https://github.com/dannagle/PacketSender/releases/latest"));
             }
-
-            settings.sync();
         }
     }
 
@@ -834,13 +912,35 @@ void MainWindow::on_packetHexEdit_lostFocus()
     ui->packetHexEdit->setText(quicktestHex);
 
 
+    QTimer::singleShot(100.0, [this](){
+        ui->packetHexEdit->deselect();
+        ui->packetHexEdit->setCursorPosition(ui->packetHexEdit->text().size());
+    });
+
+
 }
 
 
 
 void MainWindow::on_requestPathEdit_lostFocus()
 {
+    QDEBUG();
 
+    auto isHttp = ui->udptcpComboBox->currentText().toLower().contains("http");
+
+    QString quicktestURL =  ui->requestPathEdit->text().trimmed().toLower();
+    auto checkHTTP = quicktestURL.startsWith("http://") || quicktestURL.startsWith("https://");
+
+    if(isHttp && checkHTTP) {
+
+        ui->packetPortEdit->setText(QString::number(Packet::getPortFromURL(quicktestURL)));
+        ui->packetIPEdit->setText(Packet::getHostFromURL(quicktestURL));
+        ui->requestPathEdit->setText(Packet::getRequestFromURL(quicktestURL));
+        int index = ui->udptcpComboBox->findText(Packet::getMethodFromURL(quicktestURL));
+        if(index >= 0) {
+            ui->udptcpComboBox->setCurrentIndex(index);
+        }
+    }
 }
 
 void MainWindow::on_packetASCIIEdit_lostFocus()
@@ -856,6 +956,12 @@ void MainWindow::on_packetASCIIEdit_lostFocus()
     ui->packetASCIIEdit->setText(Packet::hexToASCII(quicktestASCII2));
     ui->packetASCIIEdit->setToolTip("");
 
+    QTimer::singleShot(100.0, [this](){
+        ui->packetASCIIEdit->deselect();
+        ui->packetASCIIEdit->setCursorPosition(ui->packetASCIIEdit->text().size());
+    });
+
+
 
 }
 
@@ -922,7 +1028,7 @@ void MainWindow::sendClick(QString packetName)
             if (toSend.repeat > 0) {
                 toSend.timestamp = QDateTime::currentDateTime();
 
-                stopResendingButton->setStyleSheet("QPushButton { color: green; } QPushButton::hover { color: #BC810C; } ");
+                stopResendingButton->setStyleSheet("QPushButton { color: green; background-color: #505F69; } QPushButton::hover { color: #BC810C; background-color: #505F69; } ");
                 packetsRepeat.append(toSend);
                 stopResendingButton->setText("Resending (" + QString::number(packetsRepeat.size()) + ")");
                 stopResending = 0;
@@ -940,6 +1046,17 @@ void MainWindow::sendClick(QString packetName)
 void MainWindow::on_savePacketButton_clicked()
 {
 
+
+    if(ui->packetASCIIEdit->hasFocus()) {
+        QDEBUG() << "Forcing ASCII edit trigger";
+        on_packetASCIIEdit_editingFinished();
+    }
+
+    if(ui->packetHexEdit->hasFocus()) {
+        QDEBUG() << "Forcing HEX edit trigger";
+        on_packetHexEdit_editingFinished();
+    }
+
     Packet testPacket;
     testPacket.init();
     testPacket.name = ui->packetNameEdit->text().trimmed();
@@ -1173,7 +1290,7 @@ void MainWindow::on_testPacketButton_clicked()
 
         testPacket.timestamp = QDateTime::currentDateTime();
 
-        stopResendingButton->setStyleSheet("QPushButton { color: green; } QPushButton::hover { color: #BC810C; } ");
+        stopResendingButton->setStyleSheet("QPushButton { color: green; background-color: #505F69; } QPushButton::hover { color: #BC810C; background-color: #505F69; } ");
         packetsRepeat.append(testPacket);
         stopResendingButton->setText("Resending (" + QString::number(packetsRepeat.size()) + ")");
         stopResending = 0;
@@ -1337,6 +1454,23 @@ void MainWindow::on_packetsTable_itemChanged(QTableWidgetItem *item)
         if ((newText.trimmed().toUpper() == "TCP") || (newText.trimmed().toUpper() == "UDP") || (newText.trimmed().toUpper() == "SSL")) {
             updatePacket.tcpOrUdp = newText.trimmed().toUpper();
         }
+        auto isHTTP = newText.trimmed().toUpper().contains("HTTP") || newText.trimmed().toUpper().contains("GET") || newText.trimmed().toUpper().contains("POST");
+
+        if(isHTTP) {
+            auto isHTTPS = (newText.trimmed().toUpper().contains("HTTPS"));
+            auto isPOST = newText.trimmed().toUpper().contains("POST");
+            updatePacket.tcpOrUdp = "HTTP";
+            if(isHTTPS) {
+                updatePacket.tcpOrUdp.append("S");
+            }
+            if(isPOST) {
+                updatePacket.tcpOrUdp.append(" Post");
+            } else {
+                updatePacket.tcpOrUdp.append(" Get");
+            }
+        }
+
+
     }
     if (datatype == Settings::ASCII_STR) {
         QString hex = Packet::ASCIITohex(newText);
@@ -1375,6 +1509,14 @@ void MainWindow::poodlepic()
     bruce->show();
 }
 
+void MainWindow::puppypic()
+{
+    QDEBUG();
+
+    IrisAndMarigold *pups = new IrisAndMarigold(this);
+    pups->show();
+}
+
 void MainWindow::shortcutkey1()
 {
     ui->packetNameEdit->setFocus();
@@ -1477,6 +1619,12 @@ void MainWindow::populateTableRow(int rowCounter, Packet tempPacket)
     Packet::populateTableWidgetItem(tItem, tempPacket);
     tItem->setData(Packet::DATATYPE, Settings::ASCII_STR);
 
+    if(tempPacket.isHTTP()) {
+        tItem->setText(tempPacket.requestPath);
+        tItem->setData(Packet::DATATYPE, Settings::REQUEST_STR);
+    }
+
+
     QSize tSize = tItem->sizeHint();
     tSize.setWidth(200);
     tItem->setSizeHint(tSize);
@@ -1488,6 +1636,17 @@ void MainWindow::populateTableRow(int rowCounter, Packet tempPacket)
     Packet::populateTableWidgetItem(tItem, tempPacket);
     tItem->setData(Packet::DATATYPE, Settings::HEX_STR);
     ui->packetsTable->setItem(rowCounter, packetSavedTableHeaders.indexOf(Settings::HEX_STR), tItem);
+    if(tempPacket.isHTTP()) {
+        if(tempPacket.isPOST()) {
+            tItem->setText(tempPacket.asciiString());
+        } else {
+            tItem->setText("");
+        }
+        tItem->setData(Packet::DATATYPE, Settings::ASCII_STR);
+
+    }
+
+
     //QDEBUGVAR(tempPacket.hexString);
 }
 
@@ -1516,6 +1675,7 @@ void MainWindow::packetTable_checkMultiSelected()
     QStringList buttonsList;
     buttonsList.clear();
 
+    ui->generatePanelButton->hide();
 
     foreach (checkItem, totalSelected) {
         clickedPacket = Packet::fetchTableWidgetItemData(checkItem);
@@ -1527,13 +1687,10 @@ void MainWindow::packetTable_checkMultiSelected()
         }
     }
 
-    while (packetList.size() > 1) {
-        //Multi not supported in this way anymore.
-        //Drop all but one.
-        packetList.removeLast();
+    if (packetList.size() > 1) {
+       ui->generatePanelButton->show();
     }
 
-
     ui->testPacketButton->setText("Send");
     ui->testPacketButton->setStyleSheet("");
 
@@ -1644,7 +1801,7 @@ void MainWindow::refreshTimerTimeout()
 
 
     while (maxLogSize > 0 && packetsLogged.size() > maxLogSize) {
-        packetsLogged.removeFirst();
+        packetsLogged.removeLast();
     }
 
 
@@ -1718,6 +1875,8 @@ void MainWindow::applyNetworkSettings()
         statusBarMessage("Left " + QString::number(joinedSize) + " multicast group(s)");
     }
 
+    packetsLogged.useEllipsis = settings.value("ellipsisCheck", true).toBool();
+
     packetNetwork.kill();
     packetNetwork.init();
     packetNetwork.responseData = settings.value("responseHex", "").toString().trimmed();
@@ -1733,7 +1892,7 @@ void MainWindow::applyNetworkSettings()
 
 void MainWindow::cancelResends()
 {
-    stopResendingButton->setStyleSheet("QPushButton { color: black; } QPushButton::hover { color: #BC810C; } ");
+    stopResendingButton->setStyleSheet(PersistentConnection::RESEND_BUTTON_STYLE);
     stopResendingButton->setText("Resending");
     stopResending = 1;
 }
@@ -1851,7 +2010,7 @@ void MainWindow::on_bugsLinkButton_clicked()
 
 void MainWindow::on_forumsPacketSenderButton_clicked()
 {
-    QDesktopServices::openUrl(QUrl("http://forums.packetsender.com/"));
+    QDesktopServices::openUrl(QUrl("http://forums.naglecode.com/"));
 
 
 }
@@ -1925,12 +2084,18 @@ void MainWindow::setIPMode()
     bool isIPv6 = packetNetwork.IPv6Enabled();
 
     IPmodeButton->setText(packetNetwork.getIPmode());
+    themeTheButton(IPmodeButton);
 
-    if (isIPv6) {
-        QDEBUG() << "Set IPv6 stylesheet";
-        IPmodeButton->setStyleSheet(IPv6Stylesheet);
+    QString islight = "";
+
+    if(darkMode) {
+        islight = "light";
+    }
+
+    if(isIPv6) {
+        IPmodeButton->setStyleSheet("QPushButton { color: " + islight +"blue}");
     } else {
-        IPmodeButton->setStyleSheet(IPv4Stylesheet);
+        IPmodeButton->setStyleSheet("QPushButton { color:  " + islight +"green}");
     }
 
 
@@ -2354,32 +2519,196 @@ void MainWindow::on_actionDonate_Thank_You_triggered()
 
 void MainWindow::on_udptcpComboBox_currentIndexChanged(const QString &arg1)
 {
+    auto isHttp = arg1.toLower().contains("http");
+    auto isPost = arg1.toLower().contains("post") && isHttp;
+
+    if(isHttp) {
+        ui->asciiLabel->setText("Data");
+    } else {
+        ui->asciiLabel->setText("ASCII");
+    }
 
 
-    Q_UNUSED(arg1)
+    for (int i = 0; i < ui->hexHorizLayout->count(); ++i) {
+        QWidget *w = ui->hexHorizLayout->itemAt(i)->widget();
+        if(w != nullptr) {
+            w->setVisible(!isHttp);
+        }
+    }
 
     for (int i = 0; i < ui->requestLayout->count(); ++i) {
         QWidget *w = ui->requestLayout->itemAt(i)->widget();
         if(w != nullptr) {
-            w->setVisible(false);
+            w->setVisible(isHttp);
         }
     }
+
+    for (int i = 0; i < ui->asciiLayout->count(); ++i) {
+        QWidget *w = ui->asciiLayout->itemAt(i)->widget();
+        if(w != nullptr) {
+            w->setVisible((!isHttp) || isPost);
+        }
+    }
+
+    ui->genPostDataButton->setVisible(isPost);
 }
 
 void MainWindow::on_genPostDataButton_clicked()
 {
+    PostDataGen * phttp = new PostDataGen(this, ui->packetASCIIEdit->text());
 
 
+    bool ready = connect(phttp, &PostDataGen::postGenerated, this, [=](QString val) {
+        // use action as you wish
+        QDEBUGVAR(val);
+        ui->packetASCIIEdit->setText(val);
+
+        on_packetASCIIEdit_editingFinished();
+    });
+
+    if (!ready) {
+        QDEBUG() << "postGenerated connection false";
+    }
+
+
+    phttp->show();
+
 
 }
 
 void MainWindow::on_generatePanelButton_clicked()
 {
 
+    QList<Packet> packetList;
+    QModelIndexList indexes = ui->packetsTable->selectionModel()->selectedIndexes();
+    QModelIndex index;
+    QString selected, name;
 
+    QStringList nameList;
+    nameList.clear();
+
+    foreach (index, indexes) {
+        selected = index.data(Packet::PACKET_NAME).toString();
+        if(selected.isEmpty()) {
+            continue;
+        }
+        if (!nameList.contains(selected)) {
+            nameList.append(selected);
+        }
+    }
+
+    foreach (name, nameList) {
+        Packet pkt = Packet::fetchFromDB(name);
+        packetList.append(pkt);
+    }
+
+    PanelGenerator * gpanel = new PanelGenerator(this);
+
+    QDEBUG() << " packet send connect attempt:" << connect(gpanel, SIGNAL(sendPacket(Packet)),
+             &packetNetwork, SLOT(packetToSend(Packet)));
+
+
+    gpanel->init(packetList);
+    gpanel->show();
 }
 
+
 void MainWindow::on_actionPanel_Generator_triggered()
 {
+    PanelGenerator * gpanel = new PanelGenerator(this);
+
+    QDEBUG() << " packet send connect attempt:" << connect(gpanel, SIGNAL(sendPacket(Packet)),
+             &packetNetwork, SLOT(packetToSend(Packet)));
+
+    gpanel->initAutoLaunchOrEditMode();
+    gpanel->show();
 
 }
+
+
+void MainWindow::on_actionNew_Panel_triggered()
+{
+    PanelGenerator * gpanel = new PanelGenerator(this);
+
+    QDEBUG() << " packet send connect attempt:" << connect(gpanel, SIGNAL(sendPacket(Packet)),
+             &packetNetwork, SLOT(packetToSend(Packet)));
+    gpanel->show();
+}
+
+void MainWindow::on_testPacketButton_pressed()
+{
+    if(ui->packetASCIIEdit->hasFocus()) {
+        QDEBUG() << "Forcing ASCII edit trigger";
+        on_packetASCIIEdit_editingFinished();
+    }
+
+    if(ui->packetHexEdit->hasFocus()) {
+        QDEBUG() << "Forcing HEX edit trigger";
+        on_packetHexEdit_editingFinished();
+    }
+
+}
+
+bool PreviewFilter::eventFilter(QObject *watched, QEvent *event)
+{
+    if (event->type() == QEvent::MouseButtonDblClick)
+    {
+
+        QDialog previewDlg;
+        previewDlg.setWindowFlags(previewDlg.windowFlags() & ~Qt::WindowContextHelpButtonHint);
+        previewDlg.setWindowTitle("Multi-line editor");
+        QVBoxLayout* layout = new QVBoxLayout{&previewDlg};
+        QPlainTextEdit* editor = new QPlainTextEdit{&previewDlg};
+        QLineEdit* _lineEdit = dynamic_cast<QLineEdit*>(parent());
+        _lineEdit->deselect();
+
+        QPushButton* updateButton = new QPushButton(tr("Update"));
+        QPushButton* closeButton = new QPushButton(tr("Close"));
+        QObject::connect(closeButton, &QPushButton::clicked,
+            [&previewDlg]
+            {
+                previewDlg.close();
+            });
+
+        QObject::connect(updateButton, &QPushButton::clicked,
+            [&previewDlg, &editor, &_lineEdit, this]
+            {
+                QString contents = editor->toPlainText();
+                if(_lineEdit == this->asciiEdit) {
+                    this->asciiEdit->setText(contents);
+                    emit asciiUpdated();
+                }
+                if(_lineEdit == this->hexEdit) {
+                    this->hexEdit->setText(contents);
+                    emit hexUpdated();
+                }
+                previewDlg.close();
+            });
+
+        layout->addWidget(editor);
+        layout->addWidget(updateButton);
+        layout->addWidget(closeButton);
+
+        previewDlg.setLayout(layout);
+        previewDlg.setModal(true);
+
+        // we have to make a round trip, ascii to hex and then back to ascii with whitespace
+        auto asciidata = _lineEdit->text();
+        if(_lineEdit == this->hexEdit)
+        {
+            editor->setPlainText(asciidata.simplified() + " ");
+        }
+        if(_lineEdit == this->asciiEdit)
+        {
+            //do round trip conversion
+            asciidata = Packet::ASCIITohex(asciidata);
+            auto br = Packet::HEXtoByteArray(asciidata);
+            editor->setPlainText(QString(br));
+        }
+        editor->moveCursor(QTextCursor::End);
+
+        previewDlg.exec();
+    }
+
+    return QObject::eventFilter(watched, event);
+}
