@@ -4,7 +4,7 @@
  * Licensed GPL v2
  * http://PacketSender.com/
  *
- * Copyright Dan Nagle
+ * Copyright NagleCode, LLC
  *
  */
 
@@ -18,15 +18,26 @@
 #include <QMessageBox>
 #include <QHostInfo>
 #include <QtGlobal>
+#include <QUrlQuery>
+#include <QStandardPaths>
+
 #include "settings.h"
 
 #include "persistentconnection.h"
 
-PacketNetwork::PacketNetwork(QWidget *parent) :
+#ifndef RENDER_ONLY
+#include "persistenthttp.h"
+#endif
+
+PacketNetwork::PacketNetwork(QObject *parent) :
     QObject(parent)
 {
 
     joinedMulticast.clear();
+    http = new QNetworkAccessManager(this);
+    QDEBUG() << " http connect attempt:" << connect(http, SIGNAL(finished(QNetworkReply*)),
+             this, SLOT(httpFinished(QNetworkReply*)));
+    consoleMode = false;
 }
 
 
@@ -202,9 +213,13 @@ void PacketNetwork::init()
     msgBoxBindError.setStandardButtons(QMessageBox::Ok);
     msgBoxBindError.setDefaultButton(QMessageBox::Ok);
     msgBoxBindError.setIcon(QMessageBox::Warning);
-    const QString lowPortText = "Packet Sender attempted (and failed) to bind to a UDP port [PORT], which is less than 1024. \n\nPrivileged ports requires running Packet Sender with admin-level / root permissions.";
-    const QString portConsumedText = "Packet Sender attempted (and failed) to bind to a UDP port [PORT].\n\n - Are you running multiple instances? \n\n - Trying to bind to a missing custom IP?";
+    const QString lowPortText = "Packet Sender attempted (and failed) to bind to port [PORT], which is less than 1024. \n\nPrivileged ports requires running Packet Sender with admin-level / root permissions.";
+    const QString portConsumedText = "Packet Sender attempted (and failed) to bind to port [PORT].\n\n - Are you running multiple instances? \n\n - Trying to bind to a missing custom IP?";
 
+#ifdef RENDER_ONLY
+    tcpPortList.clear();
+    sslPortList.clear();
+#endif
 
 
     QUdpSocket *udpSocket;
@@ -219,6 +234,8 @@ void PacketNetwork::init()
                               IPV4_OR_IPV6
                               , udpPort);
 
+        udpSocket->setSocketOption(QAbstractSocket::MulticastTtlOption, 128);
+
         if ((!bindResult) && (!erroronce)) {
             QDEBUGVAR(udpPort);
             erroronce = true;
@@ -284,17 +301,11 @@ void PacketNetwork::init()
         }
 
 
-
         QDEBUG() << connect(tcp, SIGNAL(packetReceived(Packet)), this, SLOT(packetReceivedECHO(Packet)))
                  << connect(tcp, SIGNAL(toStatusBar(QString, int, bool)), this, SLOT(toStatusBarECHO(QString, int, bool)))
                  << connect(tcp, SIGNAL(packetSent(Packet)), this, SLOT(packetSentECHO(Packet)));
 
 
-        QDEBUG() << connect(ssl, SIGNAL(packetReceived(Packet)), this, SLOT(packetReceivedECHO(Packet)))
-                 << connect(ssl, SIGNAL(toStatusBar(QString, int, bool)), this, SLOT(toStatusBarECHO(QString, int, bool)))
-                 << connect(ssl, SIGNAL(packetSent(Packet)), this, SLOT(packetSentECHO(Packet)));
-
-
     }
 
 
@@ -319,6 +330,11 @@ void PacketNetwork::init()
     smartList.append(Packet::fetchSmartConfig(5, SETTINGSFILE));
 
 
+#ifdef RENDER_ONLY
+    activateTCP = false;
+    activateSSL = false;
+    smartList.clear();
+#endif
     if (settings.value("delayAfterConnectCheck", false).toBool()) {
         delayAfterConnect = 500;
     }
@@ -697,6 +713,9 @@ void PacketNetwork::packetToSend(Packet sendpacket)
     sendpacket.receiveBeforeSend = receiveBeforeSend;
     sendpacket.delayAfterConnect = delayAfterConnect;
     sendpacket.persistent = persistentConnectCheck;
+    if(consoleMode) {
+        sendpacket.persistent = false;
+    }
 
     if(translateMacroSend) {
         QString data = Packet::macroSwap(sendpacket.asciiString());
@@ -799,9 +818,196 @@ void PacketNetwork::packetToSend(Packet sendpacket)
 
 
     }
+    if (sendpacket.isHTTP()) {
+        QDEBUG() << "http request" << sendpacket.requestPath;
+
+
+        // TODO: catch ssl errors!
+        // connect(http, SIGNAL(sslErrors(QNetworkReply*, const QList<QSslError> & )), this, SLOT(sslErrorsSlot(QNetworkReply*, const QList<QSslError> & )));
+
+
+        QString url = "";
+        QString portUrl = "";
+        QString portUrlS = ":" + QString::number(sendpacket.port);
+
+        if(sendpacket.isHTTPS()) {
+            url = "https://";
+            if(sendpacket.port != 443) {
+                portUrl = portUrlS;
+            }
+        } else {
+            url = "http://";
+            if(sendpacket.port != 80) {
+                portUrl = portUrlS;
+            }
+        }
+        url += sendpacket.toIP + portUrl + sendpacket.requestPath;
+
+        QHash<QString, QString> bonusHeaders = Settings::getRawHTTPHeaders(sendpacket.toIP);
+
+
+        //QDEBUGVAR(sendpacket.toIP);
+        //QDEBUGVAR(sendpacket.requestPath);
+        QDEBUGVAR(url);
+        //return;
+        QNetworkRequest request = QNetworkRequest(QUrl(url));
+
+        sendpacket.fromIP = "You";
+        sendpacket.fromPort = 0;
+        if(consoleMode) {
+            sendpacket.persistent = false;
+        }
+
+        http->setProperty("persistent", sendpacket.persistent);
+        foreach(QString key, bonusHeaders.keys()) {
+            QDEBUG()<<"Setting header" << key << bonusHeaders[key];
+            request.setRawHeader(key.toLatin1(), bonusHeaders[key].toLatin1());
+        }
+
+        if(translateMacroSend) {
+            QString data = Packet::macroSwap(sendpacket.asciiString());
+            sendpacket.hexString = Packet::ASCIITohex(data);
+        }
+
+        QByteArray bytes = sendpacket.getByteArray();
+        QString bytes_trimmed = QString(bytes.trimmed());
+
+        if(sendpacket.isPOST()) {
+            request.setHeader(QNetworkRequest::ContentTypeHeader,
+                "application/x-www-form-urlencoded");
+
+            if(Settings::detectJSON_XML()) {
+
+                if ((bytes_trimmed.startsWith("{") && bytes_trimmed.endsWith("}")) ||
+                    (bytes_trimmed.startsWith("[") && bytes_trimmed.endsWith("]")) )  {
+                    request.setHeader(QNetworkRequest::ContentTypeHeader,
+                        "application/json");
+                }
+
+
+                if (bytes_trimmed.startsWith("<") && bytes_trimmed.endsWith(">")) {
+                    request.setHeader(QNetworkRequest::ContentTypeHeader,
+                        "application/xml");
+                }
+            }
+
+
+            QDEBUG() << "http post request";
+            http->post(request, sendpacket.getByteArray());
+
+        } else {
+            QDEBUG() << "http get request";
+            http->get(request);
+        }
+
+        emit packetSent(sendpacket);
+
+    }
+
+
 
 }
 
+void PacketNetwork::httpError(QNetworkRequest* pReply)
+{
+    QDEBUGVAR(pReply);
+
+}
+
+void PacketNetwork::sslErrorsSlot(QNetworkReply *reply, const QList<QSslError> &errors)
+{
+    Packet errorPacket;
+    errorPacket.init();
+
+    QUrl url = reply->url();
+
+    int defaultPort = 80;
+    if(url.scheme().toLower().contains("https")) {
+        defaultPort = 443;
+    }
+
+    errorPacket.fromIP = url.host();
+    errorPacket.port = 0;
+    errorPacket.fromPort = static_cast<unsigned int>(url.port(defaultPort));
+    errorPacket.toIP = "You";
+    errorPacket.tcpOrUdp = "HTTP";
+
+    QDEBUGVAR(errors.size());
+    if (errors.size() > 0) {
+
+        QSslError sError;
+        foreach (sError, errors) {
+            errorPacket.hexString.clear();
+            errorPacket.errorString = sError.errorString();
+            emit packetSent(errorPacket);
+        }
+
+    }
+
+
+    reply->ignoreSslErrors();
+}
+
+void PacketNetwork::httpFinished(QNetworkReply* pReply)
+{
+
+    QByteArray data = pReply->readAll();
+    QString str = QString(data);
+    str.truncate(1000);
+    QDEBUG() << "finished http." << str;
+
+    Packet httpPacket;
+    httpPacket.init();
+
+    QUrl url = pReply->url();
+
+    QVariant status_code = pReply->attribute(QNetworkRequest::HttpStatusCodeAttribute);
+    if (status_code.isValid()) {
+        // status_code.toInt();
+        httpPacket.errorString = status_code.toString();
+    }
+
+    int defaultPort = 80;
+    if(url.scheme().toLower().contains("https")) {
+        defaultPort = 443;
+    }
+    httpPacket.fromIP = url.host();
+    httpPacket.port = 0;
+    httpPacket.fromPort = static_cast<unsigned int>(url.port(defaultPort));
+    httpPacket.toIP = "You";
+    httpPacket.hexString = Packet::byteArrayToHex(data);
+    httpPacket.tcpOrUdp = "HTTP";
+
+    if(pReply->error() != QNetworkReply::NoError) {
+        QDEBUG() << "Ended in error";
+        if(httpPacket.errorString.isEmpty()) {
+            httpPacket.errorString = pReply->errorString();
+        } else {
+            httpPacket.errorString += ", " + pReply->errorString();
+        }
+    }
+
+#ifdef RENDER_ONLY
+    http->setProperty("persistent", false);
+#endif
+
+#ifndef RENDER_ONLY
+
+    QDEBUGVAR(http->property("persistent").toBool());
+    if(http->property("persistent").toBool()) {
+        PersistentHTTP * view = new PersistentHTTP();
+        view->init(data, url);
+        view->show();
+        view->setAttribute(Qt::WA_DeleteOnClose);
+    }
+
+#endif
+    emit packetSent(httpPacket);
+
+}
+
+
+
 QList<ThreadedTCPServer *> PacketNetwork::allTCPServers()
 {
     QList<ThreadedTCPServer *> theServers;
